@@ -22,9 +22,10 @@
 [ -z "$SQUASH_INGRESS" ] && SQUASH_INGRESS="1"
 [ -z "$IQDISC_OPTS" ] && IQDISC_OPTS=""
 [ -z "$EQDISC_OPTS" ] && EQDISC_OPTS=""
-[ -z "$TC" ] && TC=$(which tc)
-#[ -z "$TC" ] && TC="sqm_logger tc"# this redirects all tc calls into the log
-[ -z "$IP" ] && IP=$(which ip)
+[ -z "$TC" ] && TC=tc_wrapper
+[ -z "$TC_BINARY" ] && TC_BINARY=$(which tc)
+[ -z "$IP" ] && IP=ip_wrapper
+[ -z "$IP_BINARY" ] && IP_BINARY=$(which ip)
 # Try modprobe first, fall back to insmod
 [ -z "$INSMOD" ] && INSMOD=$(which modprobe) || INSMOD=$(which insmod)
 [ -z "$TARGET" ] && TARGET="5ms"
@@ -35,8 +36,27 @@
 [ -z "$INGRESS_CAKE_OPTS" ] && INGRESS_CAKE_OPTS="diffserv4"
 [ -z "$EGRESS_CAKE_OPTS" ] && EGRESS_CAKE_OPTS="diffserv4"
 
+# Logging verbosity
+VERBOSITY_SILENT=0
+VERBOSITY_ERROR=1
+VERBOSITY_WARNING=2
+VERBOSITY_INFO=5
+VERBOSITY_DEBUG=8
+VERBOSITY_TRACE=10
+[ -z "$SQM_VERBOSITY" ] && SQM_VERBOSITY=$VERBOSITY_INFO
+
+[ -z "$SQM_DEBUG" ] && SQM_DEBUG=0
+if [ "$SQM_DEBUG" == 1 ]
+then
+    SQM_DEBUG_LOG=${SQM_STATE_DIR}/${IFACE}.debug.log
+    OUTPUT_TARGET=${SQM_DEBUG_LOG}
+else
+    OUTPUT_TARGET="/dev/null"
+fi
+
+
 # This is used for writing the state file
-ALL_SQM_VARS="IFACE UPLINK DOWNLINK SCRIPT ENABLED QDISC LLAM LINKLAYER OVERHEAD STAB_MTU STAB_MPU STAB_TSIZE AUTOFLOW ILIMIT ELIMIT TARGET ITARGET ETARGET IECN EECN SQUASH_DSCP SQUASH_INGRESS IQDISC_OPTS EQDISC_OPTS INGRESS_CAKE_OPTS EGRESS_CAKE_OPTS"
+ALL_SQM_VARS="IFACE UPLINK DOWNLINK SCRIPT ENABLED QDISC LLAM LINKLAYER OVERHEAD STAB_MTU STAB_MPU STAB_TSIZE AUTOFLOW ILIMIT ELIMIT TARGET ITARGET ETARGET IECN EECN SQUASH_DSCP SQUASH_INGRESS IQDISC_OPTS EQDISC_OPTS INGRESS_CAKE_OPTS EGRESS_CAKE_OPTS SQM_DEBUG SQM_DEBUG_LOG OUTPUT_TARGET"
 
 # These are the modules that do_modules() will attempt to load
 ALL_MODULES="act_ipt sch_$QDISC sch_ingress act_mirred cls_fw cls_flow cls_u32 sch_htb sch_hfsc"
