@@ -4,33 +4,73 @@
 #
 #       Copyright (C) 2012-4 Michael D. Taht, Toke Høiland-Jørgensen, Sebastian Moeller
 
-#improve the logread output
 sqm_logger() {
-    if [ "$SQM_SYSLOG" -eq "1" ]; then
-        logger -t SQM -s "$*"
-    else
-        echo "$@" >&2
+    case $1 in
+        ''|*[!0-9]*) LEVEL=$VERBOSITY_INFO ;; # empty or non-numbers
+        *) LEVEL=$1; shift ;;
+    esac
+
+    if [ "$SQM_VERBOSITY" -ge "$LEVEL" ] ; then
+	if [ "$SQM_SYSLOG" -eq "1" ]; then
+    	    logger -t SQM -s "$*"
+	else
+    	    echo "$@" >&2
+	fi
+    fi
+    #sm: slightly dangerous as this will keep adding to the log file
+    if [ -n "${SQM_DEBUG}" -a "${SQM_DEBUG}" == 1 ]; then
+        if [ "$SQM_VERBOSITY" -ge "$LEVEL" -o "$LEVEL" -eq "$VERBOSITY_TRACE" ]; then
+            echo "$@" >> ${SQM_DEBUG_LOG}
+        fi
     fi
 }
 
+sqm_error() { sqm_logger $VERBOSITY_ERROR ERROR: "$@"; }
+sqm_warn() { sqm_logger $VERBOSITY_WARNING WARNING: "$@"; }
+sqm_log() { sqm_logger $VERBOSITY_INFO "$@"; }
+sqm_debug() { sqm_logger $VERBOSITY_DEBUG "$@"; }
+sqm_trace() { sqm_logger $VERBOSITY_TRACE "$@"; }
+
+#sm: ipt needs a toggle to show the outputs for debugging (as do all users of > /dev/null 2>&1 and friends)
 ipt() {
     d=$(echo $* | sed s/-A/-D/g)
     [ "$d" != "$*" ] && {
-        iptables $d > /dev/null 2>&1
-        ip6tables $d > /dev/null 2>&1
+	sqm_trace "iptables ${d}"
+        iptables $d >> ${OUTPUT_TARGET} 2>&1
+	sqm_trace "ip6tables ${d}"
+        ip6tables $d >> ${OUTPUT_TARGET} 2>&1
     }
     d=$(echo $* | sed s/-I/-D/g)
     [ "$d" != "$*" ] && {
-        iptables $d > /dev/null 2>&1
-        ip6tables $d > /dev/null 2>&1
+	sqm_trace "iptables ${d}"
+        iptables $d >> ${OUTPUT_TARGET} 2>&1
+	sqm_trace "ip6tables ${d}"
+        ip6tables $d >> ${OUTPUT_TARGET} 2>&1
     }
-    iptables $* > /dev/null 2>&1
-    ip6tables $* > /dev/null 2>&1
+    sqm_trace "iptables $*"
+    iptables $* >> ${OUTPUT_TARGET} 2>&1
+    sqm_trace "ip6tables ${d}"
+    ip6tables $* >> ${OUTPUT_TARGET} 2>&1
 }
 
+#sm: wrapper to call tc to allow debug logging
+tc_wrapper() {
+    tc_args=$*
+    sqm_trace "${TC_BINARY} $*"
+    ${TC_BINARY} $* >> ${OUTPUT_TARGET} 2>&1
+}
+
+#sm: wrapper to call tc to allow debug logging
+ip_wrapper() {
+    ip_args=$*
+    sqm_trace "${IP_BINARY} $*"
+    ${IP_BINARY} $* >> ${OUTPUT_TARGET} 2>&1
+}
+
+
 do_modules() {
     for m in $ALL_MODULES; do
-        ${INSMOD} $m 2>/dev/null
+        [ -d /sys/modules/${m} ] && ${INSMOD} $m 2>>${OUTPUT_TARGET}
     done
 }
 
@@ -46,34 +86,29 @@ write_state_file() {
 }
 
 
-#sqm_logger "${0} IPT_MASK: ${IPT_MASK_STRING}"
-
-
-
 # find the ifb device associated with a specific interface, return nothing of no ifb is associated with IF
 get_ifb_associated_with_if() {
     local CUR_IF=$1
-    # CUR_IFB=$( tc -p filter show parent ffff: dev ${CUR_IF} | grep -o -e ifb'[[:digit:]]\+' )
-    #local CUR_IFB=$( tc -p filter show parent ffff: dev ${CUR_IF} | grep -o -e ifb'[^)]\+' )    # my editor's syntax coloration is limitied so I need a single quote in this line (between eiditor and s)
-    local CUR_IFB=$( tc -p filter show parent ffff: dev ${CUR_IF} | grep -o -E ifb'[^)\ ]+' )    # my editor's syntax coloration is limitied so I need a single quote in this line (between eiditor and s)
-    sqm_logger "ifb associated with interface ${CUR_IF}: ${CUR_IFB}"
+    # CUR_IFB=$( $TC -p filter show parent ffff: dev ${CUR_IF} | grep -o -e ifb'[[:digit:]]\+' )
+    #local CUR_IFB=$( $TC -p filter show parent ffff: dev ${CUR_IF} | grep -o -e ifb'[^)]\+' )    # my editor's syntax coloration is limitied so I need a single quote in this line (between eiditor and s)
+    local CUR_IFB=$( $TC -p filter show parent ffff: dev ${CUR_IF} | grep -o -E ifb'[^)\ ]+' )    # my editor's syntax coloration is limitied so I need a single quote in this line (between eiditor and s)
+    sqm_debug "ifb associated with interface ${CUR_IF}: ${CUR_IFB}"
     #sm: we could not detect an associated IFB for CUR_IF
-    if [ -z "${CUR_IFB}" ]; 
+    if [ -z "${CUR_IFB}" ];
     then
-	local TMP=$( tc -p filter show parent ffff: dev ${CUR_IF} )
+	local TMP=$( $TC -p filter show parent ffff: dev ${CUR_IF} )
 	if [ ! -z "${TMP}" ];
 	then
 	    #sm: oops, there is output but we failed to properly parse it? Ask for a user report
-	    sqm_logger "#---- CUT HERE ----#"
-	    sqm_logger "get_ifb_associated_with_if failed to extrect the ifb name from:"
-	    sqm_logger $( tc -p filter show parent ffff: dev ${CUR_IF} )
-	    sqm_logger "Please report this as an issue at https://github.com/tohojo/sqm-scripts"
-	    sqm_logger "Please copy and paste everything below the cut-here line into your issue report, thanks."
+	    sqm_error "#---- CUT HERE ----#"
+	    sqm_error "get_ifb_associated_with_if failed to extrect the ifb name from:"
+	    sqm_error $( $TC_BINARY -p filter show parent ffff: dev ${CUR_IF} )
+	    sqm_error "Please report this as an issue at https://github.com/tohojo/sqm-scripts"
+	    sqm_error "Please copy and paste everything below the cut-here line into your issue report, thanks."
 	else
-	    sqm_logger "Currently no ifb is associated with ${CUR_IF}, this is normal during starting of the sqm system."
+	    sqm_debug "Currently no ifb is associated with ${CUR_IF}, this is normal during starting of the sqm system."
 	fi
     fi
-    
     echo ${CUR_IFB}
 }
 
@@ -105,7 +140,7 @@ create_new_ifb_for_if() {
 #sm TODO: report failures
 create_ifb() {
     local CUR_IFB=${1}
-    $IP link add name ${CUR_IFB} type ifb #>/dev/null 2>&1    # better be verbose
+    $IP link add name ${CUR_IFB} type ifb
     ret=$?
     return $?
 }
@@ -124,7 +159,7 @@ get_ifb_for_if() {
     # if an ifb is already associated return that
     local CUR_IFB=$( get_ifb_associated_with_if ${CUR_IF} )
     [ -z "$CUR_IFB" ] && CUR_IFB=$( create_new_ifb_for_if ${CUR_IF} )
-    [ -z "$CUR_IFB" ] && sqm_logger "Could not find existing IFB for ${CUR_IF}, nor create a new IFB instead..."
+    [ -z "$CUR_IFB" ] && sqm_warn "Could not find existing IFB for ${CUR_IF}, nor create a new IFB instead..."
     echo ${CUR_IFB}
 }
 
@@ -140,7 +175,7 @@ verify_qdisc() {
     local not=
     local ifb=TMP_IFB_4_SQM
     local root_string="root"	# this works for most qdiscs
-    
+
     if [ -n "$supported" ]; then
         local found=0
         for q in $supported; do
@@ -154,10 +189,10 @@ verify_qdisc() {
 	ingress) root_string="" ;;
     esac
 
-    $TC qdisc replace dev $ifb $root_string $qdisc >/dev/null 2>&1
+    $TC qdisc replace dev $ifb $root_string $qdisc #>>${OUTPUT_TARGET} 2>&1
     res=$?
     [ "$res" = "0" ] || not="NOT "
-    sqm_logger "QDISC $qdisc is ${not}useable."
+    sqm_debug "QDISC $qdisc is ${not}useable."
     delete_ifb $ifb
     return $res
 }
@@ -170,7 +205,7 @@ get_htb_adsll_string() {
         # HTB defaults to MTU 1600 and an implicit fixed TSIZE of 256, but HTB as of around 3.10.0
         # does not actually use a table in the kernel
         ADSLL="mpu ${STAB_MPU} linklayer ${LINKLAYER} overhead ${OVERHEAD} mtu ${STAB_MTU}"
-        sqm_logger "ADSLL: ${ADSLL}"
+        sqm_debug "ADSLL: ${ADSLL}"
     fi
     echo ${ADSLL}
 }
@@ -180,7 +215,7 @@ get_stab_string() {
     if [ "${LLAM}" = "tc_stab" -a "$LINKLAYER" != "none" ];
     then
         STABSTRING="stab mtu ${STAB_MTU} tsize ${STAB_TSIZE} mpu ${STAB_MPU} overhead ${OVERHEAD} linklayer ${LINKLAYER}"
-        sqm_logger "STAB: ${STABSTRING}"
+        sqm_debug "STAB: ${STABSTRING}"
     fi
     echo ${STABSTRING}
 }
@@ -196,21 +231,25 @@ get_cake_lla_string() {
         fi
 
         STABSTRING="${STABSTRING} overhead ${OVERHEAD}"
-        sqm_logger "cake link layer adjustments: ${STABSTRING}"
+        sqm_debug "cake link layer adjustments: ${STABSTRING}"
     fi
     echo ${STABSTRING}
 }
 
 
 sqm_stop() {
-    $TC qdisc del dev $IFACE ingress 2> /dev/null
-    $TC qdisc del dev $IFACE root 2> /dev/null
-    [ -n "$CUR_IFB" ] && $TC qdisc del dev $CUR_IFB root 2> /dev/null
-    [ -n "$CUR_IFB" ] && sqm_logger "${0}: ${CUR_IFB} shaper deleted"
+    $TC qdisc del dev $IFACE ingress #2>> ${OUTPUT_TARGET}
+    $TC qdisc del dev $IFACE root #2>> ${OUTPUT_TARGET}
+    [ -n "$CUR_IFB" ] && $TC qdisc del dev $CUR_IFB root #2>> ${OUTPUT_TARGET}
+    [ -n "$CUR_IFB" ] && sqm_debug "${0}: ${CUR_IFB} shaper deleted"
 
     [ -n "$CUR_IFB" ] && ipt -t mangle -D POSTROUTING -o $CUR_IFB -m mark --mark 0x00 -g QOS_MARK_${IFACE}
-    ipt -t mangle -D POSTROUTING -o $IFACE -m mark --mark 0x00 -g QOS_MARK_${IFACE}
-    ipt -t mangle -D PREROUTING -i vtun+ -p tcp -j MARK --set-mark 0x2
+    ipt -t mangle -D POSTROUTING -o $IFACE -m mark --mark 0x00${IPT_MASK_STRING} -g QOS_MARK_${IFACE}
+    ipt -t mangle -D PREROUTING -i vtun+ -p tcp -j MARK --set-mark 0x2${IPT_MASK_STRING}
+    #sm: not sure whether we need to make this conditional or whether they are silent if the deletion does not work out
+    ipt -t mangle -D PREROUTING -i $IFACE -m dscp ! --dscp 0 -j DSCP --set-dscp-class be
+    ipt -t mangle -D PREROUTING -i $IFACE -m mark --mark 0x00${IPT_MASK_STRING} -g QOS_MARK_${IFACE}
+
     ipt -t mangle -D OUTPUT -p udp -m multiport --ports 123,53 -j DSCP --set-dscp-class AF42
     ipt -t mangle -F QOS_MARK_${IFACE}
     ipt -t mangle -X QOS_MARK_${IFACE}
@@ -218,7 +257,7 @@ sqm_stop() {
 
     [ -n "$CUR_IFB" ] && $IP link set dev ${CUR_IFB} down
     [ -n "$CUR_IFB" ] && $IP link delete ${CUR_IFB} type ifb
-    [ -n "$CUR_IFB" ] && sqm_logger "${0}: ${CUR_IFB} interface deleted"
+    [ -n "$CUR_IFB" ] && sqm_debug "${0}: ${CUR_IFB} interface deleted"
 }
 # Note this has side effects on the prio variable
 # and depends on the interface global too
@@ -289,7 +328,7 @@ get_htb_quantum() {
         CUR_QUANTUM=$((${CUR_QUANTUM} * 2))
     fi
 
-    #sqm_logger "CUR_HTB_QUANTUM: ${CUR_QUANTUM}, BANDWIDTH: ${BANDWIDTH}"
+    sqm_debug "CUR_HTB_QUANTUM: ${CUR_QUANTUM}, BANDWIDTH: ${BANDWIDTH}"
 
     echo $CUR_QUANTUM
 }
@@ -299,7 +338,7 @@ get_htb_quantum() {
 # but I fear we actually need the wire size of the whole thing not so much the MTU
 get_mtu() {
     CUR_MTU=$(cat /sys/class/net/$1/mtu)
-    #sqm_logger "IFACE: ${1} MTU: ${CUR_MTU}"
+    sqm_debug "IFACE: ${1} MTU: ${CUR_MTU}"
     echo ${CUR_MTU}
 }
 
@@ -343,13 +382,11 @@ get_flows_count() {
 get_target() {
     local CUR_TARGET=${1}
     local CUR_LINK_KBPS=${2}
-    [ ! -z "$CUR_TARGET" ] && sqm_logger "cur_target: ${CUR_TARGET} cur_bandwidth: ${CUR_LINK_KBPS}"
+    [ ! -z "$CUR_TARGET" ] && sqm_debug "cur_target: ${CUR_TARGET} cur_bandwidth: ${CUR_LINK_KBPS}"
     CUR_TARGET_STRING=
     # either e.g. 100ms or auto
     CUR_TARGET_VALUE=$( echo ${CUR_TARGET} | grep -o -e \^'[[:digit:]]\+' )
     CUR_TARGET_UNIT=$( echo ${CUR_TARGET} | grep -o -e '[[:alpha:]]\+'\$ )
-    #[ ! -z "$CUR_TARGET" ] && sqm_logger "CUR_TARGET_VALUE: $CUR_TARGET_VALUE"
-    #[ ! -z "$CUR_TARGET" ] && sqm_logger "CUR_TARGET_UNIT: $CUR_TARGET_UNIT"
 
     AUTO_TARGET=
     UNIT_VALID=
@@ -375,9 +412,9 @@ get_target() {
                     TMP_INTERVAL_STRING=$( adapt_interval_to_slow_link $TMP_TARGET_US )
                     CUR_TARGET_STRING="target ${TMP_TARGET_US}us ${TMP_INTERVAL_STRING}"
                     AUTO_TARGET="1"
-                    sqm_logger "get_target defaulting to auto."
+                    sqm_debug "get_target defaulting to auto."
                 else
-                    sqm_logger "required link bandwidth in kbps not passed to get_target()."
+                    sqm_warn "required link bandwidth in kbps not passed to get_target()."
                 fi
             fi
             # but still allow explicit use of the keyword auto for backward compatibility
@@ -390,7 +427,7 @@ get_target() {
                         CUR_TARGET_STRING="target ${TMP_TARGET_US}us ${TMP_INTERVAL_STRING}"
                         AUTO_TARGET="1"
                     else
-                        sqm_logger "required link bandwidth in kbps not passed to get_target()."
+                        sqm_warn "required link bandwidth in kbps not passed to get_target()."
                     fi
                     ;;
             esac
@@ -401,9 +438,9 @@ get_target() {
                     then
                         CUR_TARGET_STRING=""    # return nothing so the default target is not over-ridden...
                         AUTO_TARGET="1"
-                        #sqm_logger "get_target using qdisc default, no explicit target string passed."
+                        sqm_debug "get_target using qdisc default, no explicit target string passed."
                     else
-                        sqm_logger "required link bandwidth in kbps not passed to get_target()."
+                        sqm_warn "required link bandwidth in kbps not passed to get_target()."
                     fi
                     ;;
             esac
@@ -411,12 +448,11 @@ get_target() {
             then
                 if [ -z "${CUR_TARGET_VALUE}" -o -z "${UNIT_VALID}" ];
                 then
-                    [ -z "$AUTO_TARGET" ] && sqm_logger "${CUR_TARGET} is not a well formed tc target specifier; e.g.: 5ms (or s, us), or one of the strings auto or default."
+                    [ -z "$AUTO_TARGET" ] && sqm_warn "${CUR_TARGET} is not a well formed tc target specifier; e.g.: 5ms (or s, us), or one of the strings auto or default."
                 fi
             fi
             ;;
     esac
-    #    sqm_logger "target: ${CUR_TARGET_STRING}"
     echo $CUR_TARGET_STRING
 }
 
@@ -426,7 +462,7 @@ get_target() {
 adapt_target_to_slow_link() {
     CUR_LINK_KBPS=$1
     CUR_EXTENDED_TARGET_US=
-    MAX_PAKET_DELAY_IN_US_AT_1KBPS=$(( 1000 * 1000 *1540 * 8 / 1000 ))
+    MAX_PAKET_DELAY_IN_US_AT_1KBPS=$(( 1000 * 1000 * 33 * 53 * 8 / 1000 ))	# for ATM the worst case expansion including overhead seems to be 33 clls of 53 bytes each
     CUR_EXTENDED_TARGET_US=$(( ${MAX_PAKET_DELAY_IN_US_AT_1KBPS} / ${CUR_LINK_KBPS} ))    # note this truncates the decimals
     # do not change anything for fast links
     [ "$CUR_EXTENDED_TARGET_US" -lt 5000 ] && CUR_EXTENDED_TARGET_US=5000
@@ -443,7 +479,7 @@ adapt_interval_to_slow_link() {
     CUR_TARGET_US=$1
     case ${QDISC} in
         *codel)
-            CUR_EXTENDED_INTERVAL_US=$(( (100 - 5) * 1000 + ${CUR_TARGET_US} ))
+            CUR_EXTENDED_INTERVAL_US=$(( (100 - 5) * 1000 + ${CUR_TARGET_US} ))	# Note this is not following codel theory to well as target should be 5-10% of interval and the simple addition does not conserve that relationship
             echo "interval ${CUR_EXTENDED_INTERVAL_US}us"
             ;;
         pie)
@@ -472,10 +508,10 @@ get_limit() {
                                           ;;
         bfifo) [ -z "$CURLIMIT" ] && [ ! -z "$LIMIT" ] && CURLIMIT=$(( ${LIMIT} * $( cat /sys/class/net/${IFACE}/mtu ) ))    # bfifo defaults to txquelength * MTU,
                ;;
-        *) sqm_logger "${QDISC} does not support a limit"
+        *) sqm_warn "qdisc ${QDISC} does not support a limit"
            ;;
     esac
-    sqm_logger "get_limit: $1 CURLIMIT: ${CURLIMIT}"
+    sqm_debug "get_limit: $1 CURLIMIT: ${CURLIMIT}"
 
     if [ ! -z "$CURLIMIT" ]
     then
@@ -485,7 +521,6 @@ get_limit() {
 
 get_ecn() {
     CURECN=$1
-    #sqm_logger CURECN: $CURECN
     case ${CURECN} in
         ECN)
             case $QDISC in
@@ -508,10 +543,10 @@ get_ecn() {
             esac
             ;;
         *)
-            sqm_logger "ecn value $1 not handled"
+            sqm_warn "ecn value $1 not handled"
             ;;
     esac
-    #sqm_logger "get_ECN: $1 CURECN: ${CURECN} IECN: ${IECN} EECN: ${EECN}"
+    sqm_debug "get_ECN: $1 CURECN: ${CURECN} IECN: ${IECN} EECN: ${EECN}"
     echo ${CURECN}
 
 }
